#!/usr/bin/python3

import netifaces as nf
import ipaddress
from uw_lookup.client import slookup
from uw_lookup.client import get_address
import sys

networks = {
        '10.0.152.0/24' : {
            'ipv4_gateway' : '10.0.152.1',
            'ipv4_nat_gateway' : '10.0.152.254'
            },
        '10.0.153.0/24' : {
            'ipv4_gateway' : '10.0.153.1',
            'ipv4_nat_gateway' : '10.0.153.254'
            },
        '10.0.154.0/24' : {
            'ipv4_gateway' : '10.0.154.1',
            'ipv4_nat_gateway' : '10.0.154.254'
            },
        '10.0.155.0/24' : {
            'ipv4_gateway' : '10.0.155.1',
            'ipv4_nat_gateway' : '10.0.155.254'
            },
        '10.15.152.0/24' : {
            'ipv4_gateway' : '10.15.152.1',
            'ipv4_nat_gateway' : '10.15.152.254'
            },
        '10.15.153.0/24' : {
            'ipv4_gateway' : '10.15.153.1',
            'ipv4_nat_gateway' : '10.15.153.254'
            },
        '10.15.154.0/24' : {
            'ipv4_gateway' : '10.15.154.1',
            'ipv4_nat_gateway' : '10.15.154.254'
            },
        '10.15.155.0/24' : {
            'ipv4_gateway' : '10.15.155.1',
            'ipv4_nat_gateway' : '10.15.155.254'
            },
        '172.19.15.192/26' : {
            'ipv4_gateway' : '172.19.15.193',
            'ipv4_nat_gateway' : '172.19.15.254'
            },
        '172.19.152.0/24' : {
            'ipv4_gateway' : '172.19.152.1',
            'ipv4_nat_gateway' : '172.19.152.254'
            },
        '172.19.153.0/24' : {
            'ipv4_gateway' : '172.19.153.1',
            'ipv4_nat_gateway' : '172.19.153.254'
            },
        '172.19.154.0/24' : {
            'ipv4_gateway' : '172.19.154.1',
            'ipv4_nat_gateway' : '172.19.154.254'
            },
        '172.19.155.0/24' : {
            'ipv4_gateway' : '172.19.155.1',
            'ipv4_nat_gateway' : '172.19.155.254'
            },
        '172.19.50.192/26' : {
            'ipv4_gateway' : '172.19.50.193',
            'ipv4_nat_gateway' : '172.19.50.254'
            },
        '172.19.51.0/27' : {
            'ipv4_gateway' : '172.19.51.1',
            'ipv4_nat_gateway' : '172.19.51.30'
            },
        '10.10.152.0/24' :{
            'ipv4_gateway' : '10.10.152.1',
            'ipv4_nat_gateway' : '10.10.152.254'
            },
        '10.10.153.0/24' :{
            'ipv4_gateway' : '10.10.153.1',
            'ipv4_nat_gateway' : '10.10.153.254' },
        '10.10.154.0/24' :{
            'ipv4_gateway' : '10.10.154.1',
            'ipv4_nat_gateway' : '10.10.154.254'
            },
        '10.10.155.0/24' :{
            'ipv4_gateway' : '10.10.155.1',
            'ipv4_nat_gateway' : '10.10.155.254'
            },
        }

#get_default_ipv4_gateway() stolen from http://stackoverflow.com/a/6556951
def get_default_ipv4_gateway():
    import socket, struct
    """Read the default gateway directly from /proc."""
    with open("/proc/net/route") as fh:
        for line in fh:
            fields = line.strip().split()
            if fields[1] != '00000000' or not int(fields[3], 16) & 2:
                continue

            return socket.inet_ntoa(struct.pack("<L", int(fields[2], 16)))


def get_interface_info(name):
    i = {}

    nf_addr = nf.ifaddresses(name)

    i = {}
    i['name'] = name

    MAC_info  = nf_addr[nf.AF_LINK][0]
    i['MAC'] = MAC_info['addr']

    try:
        IPV4_info = nf_addr[nf.AF_INET][0]
        i['ipv4_address'] = IPV4_info['addr']
        i['ipv4_netmask'] = IPV4_info['netmask']
        i['ipv4_broadcast'] = IPV4_info['broadcast']
        network = ipaddress.IPv4Network(i['ipv4_address'] + '/' + i['ipv4_netmask'], strict=False)
        i['ipv4_gateway'] = str(list(network.hosts())[0])
        try:
            custom_properties = networks[str(network)]
            for k in custom_properties:
                i[k] = custom_properties[k]
        except:
            pass
    except:
        pass

    try:
        IPV6_info  = nf_addr[nf.AF_INET6][0]
        i['ipv6_address'] = IPV6_info['addr']
        i['ipv6_netmask'] = IPV6_info['netmask']
        i['ipv6_broadcast'] = IPV6_info['broadcast']
        network = ipaddress.IPv6Network(i['ipv6_address'] + '/' + i['ipv6_netmask'], strict=False)
        i['ipv6_gateway'] = str(list(network.hosts())[0])
        try:
            custom_properties = networks[str(network)]
            for k in custom_properties:
                i[k] = custom_properties[k]
        except:
            pass
    except:
        pass

    return i

def gen_static_config(interface):
    try:
        s = ""
        if( interface['name'] == "lo" ):
            s = "# The loopback network interface" + "\n"
            s = s + "auto lo" + "\n"
            s = s + "iface lo inet loopback" + "\n"
        else:
            s = "auto " + interface['name'] + "\n"
            s = s + "iface " + interface['name'] + " inet static" + "\n"
            s = s + "     address " + interface['ipv4_address'] + "\n"
            s = s + "     netmask " + interface['ipv4_netmask'] + "\n"
            s = s + "     broadcast " + interface['ipv4_broadcast'] + "\n"
            try:
                g = "     gateway " + interface['ipv4_nat_gateway'] + "\n"
                g = g + "     up   ip route add default via " + interface['ipv4_gateway'] + " dev " + interface['name'] + " table Intranet-SCS\n"
                g = g + "     # SCS SCS Intranets\n"
                g = g + "  post-up ip rule  add to   10.15.0.0/16    lookup Intranet-SCS\n"
                g = g + "  post-up ip rule  add to   10.0.0.0/16     lookup Intranet-SCS\n"
                g = g + "  post-up ip rule  add to   172.19.152.0/22    lookup Intranet-SCS\n"
                g = g + "  post-up ip rule  add to   172.19.15.0/24     lookup Intranet-SCS\n"
                g = g + "  #For DNS\n"
                g = g + "  post-up ip rule  add to   172.19.32.0/24     lookup Intranet-SCS\n"
                g = g + "  post-up ip rule  add to   172.19.47.0/24     lookup Intranet-SCS\n\n"
                g = g + "  #For mirror.csclub.uwaterloo.ca\n"
                g = g + "  post-up ip rule  add to   129.97.134.0/24     lookup Intranet-SCS\n\n"
                g = g + "  post-up ip rule  add to   129.97.167.0/24    lookup Intranet-SCS\n"
                g = g + "  post-up ip rule  add to   129.97.173.32/27   lookup Intranet-SCS\n"
                g = g + "  post-up ip rule  add to   129.97.15.0/24     lookup Intranet-SCS\n"
                if (interface['ipv4_address'][0:3] == "10."):
                    pass
                else:
                    pass
                g = g + "\n"
                g = g + "     # Clean up after ourselves when the interface is taken down.\n"
                g = g + "     pre-down ip flush                         table Intranet-SCS\n"
                g = g + "     pre-down ip route flush cache\n"

                s = s + g

            except:
                s = s + "     gateway " + interface['ipv4_gateway'] + "\n"
        print(s)
    except:
        pass

def main():
    try:

        #IF HAVE TWO ARGUMENTS GENERATE
        if(len(sys.argv) == 3):
            interface = {}
            target = sys.argv[1]
            interface_name = sys.argv[2]
            interface['name'] = interface_name

            subnet_id = slookup(target)
            network = ipaddress.IPv4Network(subnet_id, strict=False)

            interface['ipv4_gateway'] = str(list(network.hosts())[0])
            interface['ipv4_broadcast'] = str(network.broadcast_address)
            interface['ipv4_netmask'] = str(network.with_netmask).split('/')[1]
            interface['ipv4_address'] = str(get_address(target))

            try:
                interface['ipv4_gateway'] = networks[subnet_id]['ipv4_gateway']
                interface['ipv4_nat_gateway'] = networks[subnet_id]['ipv4_nat_gateway']
            except:
                pass

            s = "# The loopback network interface" + "\n"
            s = s + "auto lo" + "\n"
            s = s + "iface lo inet loopback" + "\n"
            print(s)

            gen_static_config(interface)

        else:
            for i in nf.interfaces():
                interface_properties = get_interface_info(i)
                gen_static_config(interface_properties)
    except:
        print("# interfaces(5) file used by ifup(8) and ifdown(8)")
        print("auto lo")
        print("iface lo inet loopback")
        print("")
        print("auto eth0")
        print("iface eth0 inet dhcp")

main()
